#include <addins/synergy/apply/ecpsapplyprepareutils.inc>

DB conflictDB
DBE objWithConflictsList
DBE attrWithConflictsList
DBE originalText
DBE currentText
DBE proposedText
DBE proposedTextEdit
DBE proposedSelected
DBE editButton
DBE updateButton
DBE cancelButton
DBE finishConflictButton
string conflictDetect_defaultRCR = ""
string conflictDetect_RCRConditionStr = ""
string objsTempFile = ""

Skip conflictDetect_RCRConditionSkip = createString
Skip attributesWithConflictsList = createString
Skip marked_objSkip = createString

Skip invalid_objSkip
bool showInvalidObjectFlag = false
string invalidObjTempFile = ""

void getProposedAttrValue(string objInConflict, attrInConflict)
{
	//clear the text
	set(proposedText, "")
	set(proposedTextEdit, "")

	//retrieve the proposed value of the attribute according to the radioBox selection
	string mid = null
	string oid = null
	ecpsSplitID(objInConflict, mid, oid)
	if !null oid then
	{
		Module m = current Module
		if null m then return
		real real_oid = realOf(oid)
		int absno = intOf(real_oid)
		Object obj = object(absno, m)
		if null obj then return
		
		//get the Radio Choice, 0 = Proposed Value, 1 = Diff With Original, 2 = Diff With Current
		int radioChoice = get proposedSelected	
		string attrValue = ""
		if (radioChoice == 1 || radioChoice == 2) then
		{
			Buffer data_buffer = create
			Buffer sourceText = create
			Buffer resultBuffer = create
			sourceText += obj.attrInConflict ""
			
			Buffer theTxtBuffer = create
			if radioChoice == 1 then
			{
				//Diff with Original
				theTxtBuffer += get originalText
			}
			else
			{
				//Diff with Current
				theTxtBuffer += get currentText
			}
			string theTxt = stringOf(theTxtBuffer)
			RichText rt
			for rt in theTxt do
			{
				data_buffer += rt.text
				if rt.last then continue
				if rt.newline then data_buffer += "\n"
			}

			diff(resultBuffer, data_buffer, sourceText, true)
			attrValue = stringOf(resultBuffer)

			delete theTxtBuffer
			delete resultBuffer
			delete sourceText
			delete data_buffer
		}
		else
		{
			//Default, proposed value
			attrValue = obj.attrInConflict
		}

		if length(attrValue) > 0 then 
		{
			set(proposedText, attrValue)
			set(proposedTextEdit, attrValue)
		}
	}
}

void editProposed(DBE theDb)
{
	//switch format to Proposed Value
	set(proposedSelected, 0)
	//disable format control
	inactive proposedSelected

	//make Proposed Value editable
	show proposedTextEdit
	hide proposedText

	//activate update and cancel buttons
	active updateButton
	active cancelButton
	//disable edit button
	inactive editButton
	
	//set proposedText and proposedTextEdit values to empty
	set(proposedText, "")
	set(proposedTextEdit, "")

	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)
	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" oid

		//update proposed text
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)
	}
}

void updateProposed(DBE theDb)
{
	//switch format back to default
	set(proposedSelected, 2)
	//enable format control
	active proposedSelected

	//make Proposed Value readonly
	show proposedText
	hide proposedTextEdit

	//activate edit button
	active editButton
	//disable update and cancel buttons
	inactive updateButton
	inactive cancelButton

	//get the string currently in proposedTextEdit
	string proptextEdit = get proposedTextEdit
	
	//set proposedText and proposedTextEdit values to empty
	set(proposedText, "")
	set(proposedTextEdit, "")

	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)
	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" oid

		//take the value currently in Proposed Value and replace the value in the module
		real_oid = realOf(oid)
		int absno = intOf(real_oid)
		Object theObject = object(absno, m)
		if null theObject then return
	
		//update proposed text
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)

		if !canModify(theObject) then
		{
			warningBox(getCsintString("csString_ConflictDetect_WarnMsg_EditObject", oid))
			return
		}
		
		Buffer thePropTxtBuffer = create
		if !null proptextEdit then
		{
			RichText rt 
			for rt in proptextEdit do
			{
				thePropTxtBuffer += rt.text
				if rt.newline then thePropTxtBuffer += "\n"
			}
		}
		string theProposedText = stringOf(thePropTxtBuffer)
		delete thePropTxtBuffer
		//determine whether the value being assigned is valid
		AttrDef ad = find(m, attr_in_conflict)
		bool isValid = attributeValue(ad, theProposedText)
		if !isValid then
			warningBox(getCsintString("csString_ConflictDetect_ErrMsg_UpdateProposed"))
		else
			theObject.attr_in_conflict = theProposedText

		//update proposed text
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)
	}
}

void cancelProposed(DBE theDb)
{
	//switch format back to default
	set(proposedSelected, 2)
	//enable format control
	active proposedSelected

	//make Proposed Value readonly
	show proposedText
	hide proposedTextEdit

	//activate edit button
	active editButton
	//disable update and cancel buttons
	inactive updateButton
	inactive cancelButton

	//set proposedText and proposedTextEdit values to empty
	set(proposedText, "")
	set(proposedTextEdit, "")

	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)
	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" oid

		//update proposed text
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)
	}
}

void showSelectedObject(DB theBox)
{
	//navigate to the selected Module's Object
	int selObj = get objWithConflictsList
	string objectStr = getColumnValue(objWithConflictsList, selObj, 1)
	if length(objectStr) > 0 then
	{
		Module m = current Module
		if null m then return
		real real_oid = realOf(objectStr)
		int absno = intOf(real_oid)
		Object objectSelected = gotoObject(absno, m)
		setFocus(m)
	}
}

void keepProposedChanges(DB theBox)
{
	//remove the attribute from the Object Attributes with Conflict list
	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)

	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return

		string mid = uniqueID(item(fullName(m)))
		string obj_in_conflict = "req" mid "_" oid

		//remove the attribute from the view
		int attr_index = get attrWithConflictsList
		delete(attrWithConflictsList, attr_index)
		//remove the attribute from the attributesWithConflictsList
		Skip attrList = createString
		if !find(attributesWithConflictsList, obj_in_conflict, attrList) then return

		string theattr = null
		if find(attrList, attr_in_conflict, theattr) then
		{
			delete(attrList, attr_in_conflict)
			if noElems(attrWithConflictsList) < 1 then
			{
				delete(attributesWithConflictsList, obj_in_conflict)
				//remove the object from the view
				int obj_index = get objWithConflictsList
				delete(objWithConflictsList, obj_index)
				//remove from marked_objSkip if exists
				string tempObj = null
				if find(marked_objSkip, oid, tempObj) then
					delete(marked_objSkip, oid)
			}
		}

		//make sure to empty all text fields
		set(originalText, "")
		set(currentText, "")
		set(proposedText, "")
		set(proposedTextEdit, "")
	}
}

void discardProposedChanges(DB theBox)
{
	//discard the proposed changes for the selected object's attribute
	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)

	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return

		string mid = uniqueID(item(fullName(m)))
		string obj_in_conflict = "req" mid "_" oid

		real real_oid = realOf(oid)
		int absno = intOf(real_oid)
		Object theObject = object(absno, m)
		if null theObject then return
		if !canModify(theObject) then
		{
			warningBox(getCsintString("csString_ConflictDetect_WarnMsg_EditObject", oid))
			return
		}

		//Buffer theCurTxtBuffer = create
		//RichText rt
		//string curTxt = get currentText
		//for rt in curTxt do
		//{
		//	theCurTxtBuffer += rt.text
		//	if rt.newline then theCurTxtBuffer += "\n"
		//}
		//string theCurrentText = stringOf(theCurTxtBuffer)
		//delete theCurTxtBuffer
		//theObject.attr_in_conflict = theCurrentText
		string modifiedAttr = null
		History h
		for h in session(theObject) do
		{
			if (h.type == modifyObject) then
			{
				modifiedAttr = h.attrName
				if(attr_in_conflict == modifiedAttr) then
				{
					string theObjectValue = ""
					theObjectValue = h.oldValue
					theObject.attr_in_conflict = theObjectValue
					break
				}
			}
		}

		//store the object and attribute in temp file
		if length(objsTempFile) < 1 then
			objsTempFile = csintUserTempDir "\\" mid "_discard.dat"
		
		Stream outStream = append(objsTempFile, CP_UTF8)
		if null outStream then return

		Buffer objsBuffer = create
		objsBuffer += obj_in_conflict "," attr_in_conflict "\n"
		outStream << objsBuffer
		delete objsBuffer
		close outStream

		//remove the attribute from the view
		int attr_index = get attrWithConflictsList
		delete(attrWithConflictsList, attr_index)
		//remove the attribute from the attributesWithConflictsList
		Skip attrList = createString
		if !find(attributesWithConflictsList, obj_in_conflict, attrList) then return
			
		string theattr = null
		if find(attrList, attr_in_conflict, theattr) then
		{	
			delete(attrList, attr_in_conflict)
			if noElems(attrWithConflictsList) < 1 then
			{
				delete(attributesWithConflictsList, obj_in_conflict)
				//remove the object from the view
				int obj_index = get objWithConflictsList
				delete(objWithConflictsList, obj_index)
				//remove from marked_objSkip if exists
				string tempObj = null
				if find(marked_objSkip, oid, tempObj) then
					delete(marked_objSkip, oid)
			}
		}

		//make sure to empty all text fields
		set(originalText, "")
		set(currentText, "")
		set(proposedText, "")
		set(proposedTextEdit, "")
	}
}

Skip getMRCAttributes(OleAutoObj obj_csUser, OleAutoObj obj_csApiLib, string curmod_uniqueID)
{
	// query for the RCR
	Skip resultSkip = createString
	string errMsg = null

	string rcrID = "problem_number='" conflictDetect_defaultRCR "'"
	OleAutoObj rcrQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "Data_Report_CR", rcrID, CSINT_RCP_DIALOG_DEFAULT_CR_ATTRS)
	OleAutoObj csObjectVector = getObjectVector(rcrQueryData, 0, errMsg)

	string rcrName = getRecordField(csObjectVector, "name", errMsg)
	string rcrSubsystem = getRecordField(csObjectVector, "subsystem", errMsg)
	if !null rcrQueryData then cleanupQueryData(rcrQueryData)

	string fpName = rcrSubsystem "/problem/" rcrName "/1"
	string mrcQuery = "is_dynamic_attrs_new_of('" fpName "')"
	OleAutoObj mrcQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "Data_Report_dynamic_attrs_new", mrcQuery, CSINT_RC_NEW_ATTRS)
	int num_of_mrcs = 0
	if (!null mrcQueryData) then num_of_mrcs = get_int_from_OleAutoObj(mrcQueryData, "DataSize")
	
	int index
	for (index = 0; index< num_of_mrcs; index++)
	{
		OleAutoObj rcObjectVector = getObjectVector(mrcQueryData, index, errMsg)
		Skip mrcAttrs = objectVectorToAttrSkip(rcObjectVector, conflictDetect_defaultRCR)
		string doorsID = getAttrValueFromAttrSkip(mrcAttrs, "doorsID")
		string moduleID = null
		string objectID = null
		ecpsSplitID(doorsID, moduleID, objectID)
		if (!null moduleID) then
		{
			//determine if it is the matching one
			if (curmod_uniqueID == moduleID) then
			{
				return mrcAttrs	
			}
		}
	}
	return resultSkip
}

void clearObsoleteFlag(DB theBox)
{
	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Module m = current Module
	if null m then return
	string curmod_uniqueID = uniqueID(item(fullName(m)))
	Skip mrc_attrs = getMRCAttributes(obj_csUser, obj_csApiLib, curmod_uniqueID)
	string cvID = getAttrValueFromAttrSkip(mrc_attrs, "cvid")
	if length(cvID) > 0 then
	{
		// clear the obsolete flag and obsolete information for the default RCR
		string error_Msg = null
		error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_IS_OBSOLETE, "boolean", "false")
		csintDebug_("Result of reset isMrcObsolete to FALSE = " error_Msg)
		error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBSOLETEINFO, "text", "")
		csintDebug_("Result of reset mrcObsoleteInfo value = " error_Msg)
		error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBSINFO_RCRS, "text", "")
		csintDebug_("Result of reset mrcObsInfo_RCRs value = " error_Msg)
		error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBJSMARKED, "text", "")
		csintDebug_("Result of reset mrcObjsMarked value = " error_Msg)
		
		delete marked_objSkip
		//explicitly close the conflict dialog box
		if (!null attributesWithConflictsList) then
			cleanTwoLevelSkip(attributesWithConflictsList)
		delete conflictDetect_RCRConditionSkip
		//save the module
		if !null m then save(m)
		release theBox	
		hide theBox
	}
}

void getObjectsAttributes(DBE objectSelected, int selObj)
{
	//make sure to empty the list first
	empty attrWithConflictsList
	//make sure to empty all text fields
	set(originalText, "")
	set(currentText, "")
	set(proposedText, "")
	set(proposedTextEdit, "")

	Buffer objectStr = create
	objectStr += getColumnValue(objWithConflictsList, selObj, 1)

	//go through the skip attributesWithConflictsList
	if (!null attributesWithConflictsList && length(objectStr) > 0) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" stringOf(objectStr)
		Skip attributeList = createString
		if find(attributesWithConflictsList, obj_in_conflict, attributeList) then
		{
			int attr_count = 0
			string attrSkip
			for attrSkip in attributeList do
			{
				string attr = (string key attributeList)
				insert(attrWithConflictsList, attr_count, attr)
				attr_count++
			}
		}
	}

	delete objectStr
}

void deselectObjWithConflict(DBE objectSelected, int selObj)
{
	return
}

void activateObjWithConflict(DBE objectSelected, int selObj)
{
	return
}

void selectObjToResolve(DBE objectSelected, int selObj)
{
	//update marked_objSkip
	string theObjId = getColumnValue(objWithConflictsList, selObj, 1)
	string theObject = null
	bool isChecked = getCheck(objWithConflictsList, selObj)
	if !isChecked then
	{
		//false, remove from marked_objSkip
		if find(marked_objSkip, theObjId, theObject) then
			delete(marked_objSkip, theObjId)
	}
	else	
	{
		//true, add to marked_objSkip
		if !find(marked_objSkip, theObjId, theObject) then
			put(marked_objSkip, theObjId, theObjId)
	}

	//go through each item in objWithConflictsList
	bool allChecked = true
	int i=0
	for (i=0; i<noElems(objWithConflictsList); i++)
	{
		allChecked = getCheck(objWithConflictsList, i)
		if !allChecked then
			break
	}
	//set inactive or active
	if !allChecked then
		inactive finishConflictButton
	else
		active finishConflictButton
}

void getOriginalAttrValue(string objInConflict, attrInConflict)
{
	//clear the text
	set(originalText, "")

	//retrieve the original value of the attribute
	Module m = current Module
	if null m then return
	string mid =  uniqueID(item(fullName(m)))

	string mrc_new_cvid = null
	string mrc_orig_cvid = null
	string cachedRcpID = null
	getModuleRcCvidsFromCache(mid, cachedRcpID, mrc_new_cvid, mrc_orig_cvid)
	string rcID_selected = cachedRcpID "_" objInConflict

	string parentDir = LOADING_CACHE_DIR "\\csint_" mid "\\"
	string hFile = parentDir LOADING_MRC_ORIG_FILE
	if !fileExists(hFile) then return

	Stream inputOrig = read(hFile, CP_UTF8)
	Buffer origStreamBuffer = create
	Buffer origOrcBuffer = create
	
	Skip attrSkip = createString
	while(readSectionFromFile(inputOrig, origStreamBuffer, H_X_ORC, origOrcBuffer))
	{
		int sectionPos = 0
		string rcID = ""
		if (getValueBetweenTags(sectionPos, origOrcBuffer, "rcID", rcID)) then
		{
			if (rcID_selected == rcID) then
			{
				parseXMLforOrigValues(origOrcBuffer, attrSkip, true)
				break
			}
		}
	}

	string obj
	for obj in attrSkip do
	{
		string attr = (string key attrSkip)
		if (attrInConflict == attr) then
		{
			string objOrigValue = ""
			objOrigValue = obj
			set(originalText, objOrigValue)
			break
		}
	}

	delete origStreamBuffer
	delete origOrcBuffer

	//close the loading stream
	close inputOrig
}

void getCurrentAttrValue(string objInConflict, attrInConflict)
{
	//clear the text
	set(currentText, "")
	
	//retrieve the current value of the attribute
	string mid = null
	string oid = null
	ecpsSplitID(objInConflict, mid, oid)
	if null oid then return

	Module m = current Module
	if null m then return
	real real_oid = realOf(oid)
	int absno = intOf(real_oid)
	Object theObject = object(absno, m)
	if null theObject then return

	string modifiedAttr = null
	History h
	for h in session(theObject) do
	{
		if (h.type == modifyObject) then
		{
			modifiedAttr = h.attrName
			if (attrInConflict == modifiedAttr) then
			{
				string theObjectValue = ""
				theObjectValue = h.oldValue
				set(currentText, theObjectValue)
				break
			}
		}
	}	
}

void changeProposedText(DBE radioSelected)
{
	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	string oid = getColumnValue(objWithConflictsList, selObj, 1)

	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" oid
		
		//update proposed text
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)
	}
}

void displayRequiredValues(DBE attributeSelected)
{
	//function for retrieving the original, current, proposed values
	string attr_in_conflict = get attrWithConflictsList
	int selObj = get objWithConflictsList
	Buffer oid = create
	oid += getColumnValue(objWithConflictsList, selObj, 1)
	
	if ( length(oid) > 0 && length(attr_in_conflict) > 0 ) then
	{
		Module m = current Module
		if null m then return
		string obj_in_conflict = "req" uniqueID(item(fullName(m))) "_" stringOf(oid)

		//original
		getOriginalAttrValue(obj_in_conflict, attr_in_conflict)
	
		//current
		getCurrentAttrValue(obj_in_conflict, attr_in_conflict)
	
		//proposed
		getProposedAttrValue(obj_in_conflict, attr_in_conflict)
	}

	delete oid
}

void updateMrcObjsMarked()
{
	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	Module m = current Module
	if null m then return
	string curmod_uniqueID = uniqueID(item(fullName(m)))
	Skip mrc_attrs = getMRCAttributes(obj_csUser, obj_csApiLib, curmod_uniqueID)
	string cvID = getAttrValueFromAttrSkip(mrc_attrs, "cvid")
	if length(cvID) > 0 then
	{	
		string error_Msg = null

		//check if the objWithConflictsList listview is empty
		int num_objWithConflictsList = noElems(objWithConflictsList)
		if num_objWithConflictsList < 1 then
		{
			if showInvalidObjectFlag then return

			//reset obsolete flag
			error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_IS_OBSOLETE, "boolean", "false")
			csintDebug_("Result of reset isMrcObsolete to FALSE = " error_Msg)
			error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBSOLETEINFO, "text", "")
			csintDebug_("Result of reset mrcObsoleteInfo value = " error_Msg)
			error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBSINFO_RCRS, "text", "")
			csintDebug_("Result of reset mrcObsInfo_RCRs value = " error_Msg)
			error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBJSMARKED, "text", "")
			csintDebug_("Result of reset mrcObjsMarked value = " error_Msg)

			//save the module
			if !null m then save(m)
		}
		else
		{
			Buffer listOfObjects = create
			string obj_id
			for obj_id in marked_objSkip do
			{
				if length(obj_id) > 0 then
				{
					if length(listOfObjects) > 0 then
						listOfObjects += "_"
					listOfObjects += obj_id
				}
			}
			
			//set value of mrcObjsMarked
			error_Msg = setRCAttrValue(obj_csApiLib, obj_csUser, cvID, CSINT_RCM_RC_OBJSMARKED, "text", stringOf(listOfObjects))
			csintDebug_("Result of set mrcObjsMarked value = " error_Msg)

			delete listOfObjects
		}
	}
}

void onClose_ConflictDetect(DB theDB)
{
	Module m = current Module
	if !null m then
	{
		//update mrcObjsMarked value
		updateMrcObjsMarked()
	}
	delete marked_objSkip
	//explicitly close the conflict dialog box
	if (!null attributesWithConflictsList) then
		cleanTwoLevelSkip(attributesWithConflictsList)
	delete conflictDetect_RCRConditionSkip

	release theDB	
	hide theDB
}

void checkFinishButtonStatus(OleAutoObj obj_csUser, OleAutoObj obj_csApiLib, string curmod_uniqueID)
{
	Skip mrc_attrs = getMRCAttributes(obj_csUser, obj_csApiLib, curmod_uniqueID)
	string cvID = getAttrValueFromAttrSkip(mrc_attrs, "cvid")
	if length(cvID) > 0 then
	{
		Buffer marked_objs = create
		marked_objs = getAttrValueFromAttrSkip(mrc_attrs, CSINT_RCM_RC_OBJSMARKED)
		
		if length(marked_objs) < 1 then
		{
			inactive finishConflictButton	
			delete marked_objs
			return
		}

		//gather the list of marked rcrs
		Buffer results = create
		while(true)
		{
			versionTwoGetNextElementUnderscore(results, marked_objs)
			string themarked_obj = stringOf(results)
			if length(results) > 0 then
			{
				put(marked_objSkip, themarked_obj, themarked_obj)
			}
			if length(marked_objs) < 1 then break
		}
		
		//go through each item in objWithConflictsList
		bool allChecked = true
		Buffer item = create
		int i
		for (i=0; i<noElems(objWithConflictsList); i++)
		{
			item = getColumnValue(objWithConflictsList, i, 1)
			string tempItem = null
			if !find(marked_objSkip, stringOf(item), tempItem) then 
			{
				setCheck(objWithConflictsList, i, false)
				allChecked = false
				continue
			}
			setCheck(objWithConflictsList, i, true)
		}

		delete results
		delete item
		delete marked_objs

		if !allChecked then
			inactive finishConflictButton
		else
			active finishConflictButton
	}
	else
		inactive finishConflictButton
}

bool isMRCObsolete(OleAutoObj obj_csUser, OleAutoObj obj_csApiLib, OleAutoObj com, string curmod_uniqueID)
{
	bool result = false
	
	Skip mrc_attrs = getMRCAttributes(obj_csUser, obj_csApiLib, curmod_uniqueID)
	string cvID = getAttrValueFromAttrSkip(mrc_attrs, "cvid")
	if length(cvID) > 0 then
	{
		string is_Obsolete = getAttrValueFromAttrSkip(mrc_attrs, CSINT_RCM_RC_IS_OBSOLETE)
		if (is_Obsolete == "TRUE") then
		{
			Buffer obs_rcrs = create
			obs_rcrs = getAttrValueFromAttrSkip(mrc_attrs, CSINT_RCM_RC_OBSINFO_RCRS)
			if length(obs_rcrs) > 0 then
			{
				//generate the value conflictDetect_RCRConditionStr
				Buffer condition_buffer = create
				Buffer results = create
				while(true)
				{
					versionTwoGetNextElementUnderscore(results, obs_rcrs)
					string conflict_rcr = stringOf(results)
					put(conflictDetect_RCRConditionSkip, conflict_rcr, conflict_rcr)
					if length(results) > 0 then
					{
						if length(condition_buffer) < 1 then
							condition_buffer += "problem_number='" conflict_rcr "'"
						else
							condition_buffer += " or problem_number='" conflict_rcr "'"
					}
					if length(obs_rcrs) < 1 then break
				}
				if length(condition_buffer) > 0 then
					conflictDetect_RCRConditionStr = stringOf(condition_buffer)
				else
					conflictDetect_RCRConditionStr = ""

				delete results
				delete condition_buffer
			}
			delete obs_rcrs
			result = true
		}
	}
	
	return result
}

void checkForConflicts(string againstThisRcr, 
		       Skip aCsObjectVector, aAttrs, aObjActions,  
		       Skip allRCsInOneModule, allRCsNewAttrsInOneModule, allRCsActionsInOneModule,
		       Skip &objsToReturn)
{
	csintDebug_("checkForConflicts() start...")
	int errorAttr = 0;

	// read mem RC properties
	string aCvID = getAttrValueFromAttrSkip(aCsObjectVector, "rcID")
	string aStatus      = getAttrValueFromAttrSkip(aCsObjectVector, CSINT_RCM_RC_IS_APPLIED)
	string aRcIsRejected = getAttrValueFromAttrSkip(aCsObjectVector, CSINT_RCM_RC_IS_REJECTED)
	string aChangeType  = getAttrValueFromAttrSkip(aCsObjectVector, CSINT_RCM_RC_CHANGETYPE)
	
	//skip the mem RC if it has been applied
	if (aStatus == CSINT_RCM_RC_BEEN_APPLIED) then return
	if (aRcIsRejected == "TRUE") then return
	if !changeTypeIsRecording(aChangeType) then return
	
	string aParent_rcrID = getAttrValueFromAttrSkip(aCsObjectVector, "parent_rcrID")
	string aDoorsID     = getAttrValueFromAttrSkip(aCsObjectVector, CSINT_RCM_RC_DOORSID)
	string aModuleID = null
	string aObjRecID = null
	ecpsSplitID(aDoorsID, aModuleID, aObjRecID)
	csintDebug_("rcID = " aCvID ", doorsID = " aDoorsID ", changeType = " aChangeType ", applyStatus = " aStatus ", rejected = " aRcIsRejected)
	
	// ===========================================
	// check for attribute modification conflicts
	// looping through each ORC
	// ===========================================
	int conflictedAttrCounter = 0
	Skip attrConflicts = createString
	Skip bCsObjectVector
	for bCsObjectVector in allRCsInOneModule do
	{
		string bCvID    = getAttrValueFromAttrSkip(bCsObjectVector, "rcID")
		string bParent_rcrID = getAttrValueFromAttrSkip(bCsObjectVector, "parent_rcrID")
		string bStatus  = getAttrValueFromAttrSkip(bCsObjectVector, CSINT_RCM_RC_IS_APPLIED)
		string bRcIsRejected = getAttrValueFromAttrSkip(bCsObjectVector, CSINT_RCM_RC_IS_REJECTED)
		string bChangeType  = getAttrValueFromAttrSkip(bCsObjectVector, CSINT_RCM_RC_CHANGETYPE)

		if (bParent_rcrID == aParent_rcrID) then continue
		if (bStatus == CSINT_RCM_RC_NOT_APPLIED) then continue
		if (bRcIsRejected == "TRUE") then continue
		if !changeTypeIsRecording(bChangeType) then continue
		if bParent_rcrID != againstThisRcr then continue

		if (aCvID != bCvID) then
		{
			string bDoorsID = getAttrValueFromAttrSkip(bCsObjectVector, CSINT_RCM_RC_DOORSID)
			if (aDoorsID == bDoorsID) then //for possible conflicts on the same object
			{
				Skip bAttrs = null
				Skip bObjActions = null
				find(allRCsNewAttrsInOneModule, bCvID, bAttrs)
				find(allRCsActionsInOneModule, bCvID, bObjActions)

				// ----------------------------------------------------------
				// check attribute modify conflict if both a and b has modify
				// ----------------------------------------------------------
				checkAttributeModificationConflict(bCvID, aDoorsID, bCsObjectVector, aObjActions, bObjActions, aAttrs, bAttrs, conflictedAttrCounter, attrConflicts)
			}
		}
	}
	if (conflictedAttrCounter > 0) then
	{
		//make sure this part works
		string temp_objID = null
		if !find(objsToReturn, aDoorsID, temp_objID) then
			put(objsToReturn, aDoorsID, aDoorsID)

		Skip cur_attr_in_conflicts = createString
		find(attributesWithConflictsList, aDoorsID, cur_attr_in_conflicts)

		Skip attr_in_conflicts = createString
		Skip conflictedORCs
		for conflictedORCs in attrConflicts do
		{
			string aName = (string key attrConflicts)
			string temp_aName = null
			if !find(cur_attr_in_conflicts, aName, temp_aName) then
				put(cur_attr_in_conflicts, aName, aName)
		}
		Skip the_attr_list = createString
		if !find(attributesWithConflictsList, aDoorsID, the_attr_list) then
			put(attributesWithConflictsList, aDoorsID, cur_attr_in_conflicts)
	}

	attrConflicts = cleanTwoLevelSkip(attrConflicts)
	csintDebug_("checkForConflicts() end.")
}

void getAllObjectsWithConflicts(OleAutoObj obj_csUser, OleAutoObj obj_csApiLib, OleAutoObj com, string curmod_uniqueID)
{
	string errMsg = null
	Skip objsToReturn = createString

	string queryString = "problem_number='" conflictDetect_defaultRCR "'"
	OleAutoObj rcrQueryData = do_csApiLib_QueryStringData(obj_csApiLib, obj_csUser, "Data_Report_CR", queryString, CSINT_RCP_DIALOG_DEFAULT_CR_ATTRS)
	OleAutoObj csObjectVector = getObjectVector(rcrQueryData, 0, errMsg)

	string rcrStatus    = getRecordField(csObjectVector, "crstatus",  errMsg)
	string rcrName      = getRecordField(csObjectVector, "name",      errMsg)
	string rcrSubsystem = getRecordField(csObjectVector, "subsystem", errMsg)
	if !null rcrQueryData then cleanupQueryData(rcrQueryData)

	Skip memRCsbyModule = createString			// mid -> ( rcID -> orc attrs )
	Skip memRCsNewAttrsbyModule = createString	// mid -> ( rcID -> new attrs )
	Skip memRCsActionsbyModule = createString	// mid -> ( rcID -> actions   )
	Skip memRCsOrderByModule = createString		// mid -> ( i -> rcID )
	Skip memAnalystDataByModule = createString	// mid -> value skip

	Skip mrcCvids = createString			// mid -> cvid
	Skip mrcOrigCvids = createString		// mid -> cvid

	string fourPartName = rcrSubsystem "/problem/" rcrName "/1"
	int numMRCs = prepareRcrData("", conflictDetect_defaultRCR, fourPartName, obj_csApiLib, obj_csUser, com, mrcCvids, mrcOrigCvids, memRCsbyModule, memRCsOrderByModule, memAnalystDataByModule, memRCsNewAttrsbyModule, memRCsActionsbyModule)
	if numMRCs > 0 then
	{
		//prepareScopeRcrData
		Skip allRCsbyModule = createString
		Skip allRCsNewAttrsbyModule = createString
		Skip allRCsActionsbyModule = createString
		Skip scopeRCRs = createString
		Module currentModule = current Module
		string appliedQuery = buildAppliedStateQuery(currentModule)
		prepareScopeRcrData(false, conflictDetect_defaultRCR, appliedQuery, conflictDetect_RCRConditionSkip, memRCsbyModule, allRCsbyModule, allRCsNewAttrsbyModule, allRCsActionsbyModule, obj_csApiLib, obj_csUser, com, true, scopeRCRs) 
		
		string rcrInScope
		for rcrInScope in scopeRCRs do
		{
			Skip memRCsInOneModule
			for memRCsInOneModule in memRCsbyModule do	// loop m
			{
				string mID = (string key memRCsbyModule)
				if mID != curmod_uniqueID then continue

				Skip memRCsNewAttrsInOneModule = null
				Skip memRCsActionsInOneModule = null
				find(memRCsNewAttrsbyModule, mID, memRCsNewAttrsInOneModule)
				find(memRCsActionsbyModule, mID, memRCsActionsInOneModule) 

				Skip allRCsInOneModule
				Skip allRCsNewAttrsInOneModule
				Skip allRCsActionsInOneModule
				if find(allRCsbyModule, mID, allRCsInOneModule) then
				{
					find(allRCsNewAttrsbyModule, mID, allRCsNewAttrsInOneModule)
					find(allRCsActionsbyModule, mID, allRCsActionsInOneModule)

					Skip aCsObjectVector
					for aCsObjectVector in memRCsInOneModule do
					{
						string aRcID = getAttrValueFromAttrSkip(aCsObjectVector, "rcID")
						Skip aAttrs = null
						Skip aObjActions = null
						find(memRCsNewAttrsInOneModule, aRcID, aAttrs)
						find(memRCsActionsInOneModule, aRcID, aObjActions)

						//check for conflicts
						checkForConflicts(rcrInScope, aCsObjectVector, aAttrs, aObjActions, allRCsInOneModule, allRCsNewAttrsInOneModule, allRCsActionsInOneModule, objsToReturn)
					}
				}
			}
		}
	}
	
	//determine if there are invalid objects 
	if fileExists(invalidObjTempFile) then 
	{
		//read temp file
		Buffer fBuffer = create
		string f = readFile(invalidObjTempFile)
		fBuffer = f
		Buffer results = create
		while (true)
		{
			versionTwoGetNextElementUnderscore(results, fBuffer)
			string invalidobj = stringOf(results)
			if length(results) > 0 then put(invalid_objSkip, invalidobj, invalidobj)
			if length(fBuffer)  < 1 then break
		}	
		delete fBuffer
		delete results
	}

	//fill objWithConflictsList to display in Conflict Resolution dialog box
	string obj
	int obj_count = 0
	for obj in objsToReturn do
	{
		//display in objWithConflictsList view
		string mid = null
		string oid = null
		ecpsSplitID(obj, mid, oid)
		if null oid then continue

		if fileExists(invalidObjTempFile) then
		{
			if find(invalid_objSkip, oid) then
			{
				if !showInvalidObjectFlag then
				{
					showInvalidObjectFlag = true
					warningBox(getCsintString("csString_ConflictDetect_WarnMsg_InvalidObject"))
				}
				continue
			}
		}

		//insert object ID in listView
		insert(objWithConflictsList, obj_count, "")
		set(objWithConflictsList, obj_count, 1, oid)
		setCheck(objWithConflictsList, obj_count, false)
		
		obj_count++
	}
	if !null invalid_objSkip then delete invalid_objSkip
	if fileExists(invalidObjTempFile) then deleteFile(invalidObjTempFile)
}

void conflictDetectDialogBase(OleAutoObj obj_csUser, OleAutoObj obj_csApiLib, OleAutoObj com, string curmod_uniqueID)
{
	csintDebug_("conflictDetectDialogBase() start...")

	//create object list and attribute list
	string emptyList[] = {}

	DBE invisibleFrame2 = frame(conflictDB, "")
	invisibleFrame2->"bottom"->"unattached"

	//create label
	DBE objWithConflictsLabel = label(conflictDB, getCsintString("csString_ConflictDetect_ObjListTitle"))
	objWithConflictsLabel->"top"->"inside"->invisibleFrame2
	objWithConflictsLabel->"left"->"inside"->invisibleFrame2

	objWithConflictsList = listView(conflictDB, listViewOptionCheckboxes, 300, 4, emptyList)
	objWithConflictsList->"top"->"flush"->objWithConflictsLabel
	objWithConflictsList->"right"->"unattached"
	objWithConflictsList->"left"->"inside"->invisibleFrame2
	objWithConflictsList->"bottom"->"inside"->invisibleFrame2

	attrWithConflictsList = list(conflictDB, getCsintString("csString_ConflictDetect_AttrListTitle"), 300, 8, emptyList)
	attrWithConflictsList->"left"->"flush"->objWithConflictsList
	attrWithConflictsList->"top"->"aligned"->objWithConflictsList
	attrWithConflictsList->"bottom"->"inside"->invisibleFrame2
	attrWithConflictsList->"right"->"inside"->invisibleFrame2

	objWithConflictsLabel->"top"->"form"
	attrWithConflictsList->"top"->"form"

	DBE invisibleFrame = frame(conflictDB, "")
	invisibleFrame->"bottom"->"form"

	//create the original, current text fields
	originalText = richText(conflictDB, getCsintString("csString_ConflictDetect_Text_OriginalTitle"), "", 100, true)
	currentText = richText(conflictDB, getCsintString("csString_ConflictDetect_Text_CurrentTitle"), "", 100, true)

	originalText->"left"->"inside"->invisibleFrame
	originalText->"top"->"inside"->invisibleFrame
	originalText->"right"->"inside"->invisibleFrame
	currentText->"left"->"aligned"->originalText
	currentText->"right"->"inside"->invisibleFrame

	//create the proposed text field
	DBE propTextLbl = label(conflictDB, getCsintString("csString_ConflictDetect_Text_ProposedTitle"))
	propTextLbl->"left"->"inside"->invisibleFrame

	//RadioBox selection for proposed text
	string proposedTypeChoices [] = {getCsintString("csString_ConflictDetect_Text_ProposedTitle"), getCsintString("csString_ConflictDetect_Text_DiffWithOrig"), getCsintString("csString_ConflictDetect_Text_DiffWithCurr")}
	proposedSelected = radioBox(conflictDB, getCsintString("csString_ConflictDetect_Text_ProposedFormat"), proposedTypeChoices, 2)
	
	//create empty label
	DBE emptyLabel = label(conflictDB, "")
	emptyLabel->"left"->"inside"->invisibleFrame

	//create update, edit, cancel button
	editButton = button(conflictDB, getCsintString("csString_ConflictDetect_Button_Edit"), editProposed, false)
	updateButton = button(conflictDB, getCsintString("csString_ConflictDetect_Button_Update"), updateProposed, false)
	cancelButton = button(conflictDB, getCsintString("csString_ConflictDetect_Button_Cancel"), cancelProposed, false)
	editButton->"left"->"inside"->invisibleFrame
	updateButton->"left"->"inside"->invisibleFrame
	cancelButton->"left"->"inside"->invisibleFrame

	//readonly rich text
	proposedText = richText(conflictDB, "", "", 100, 160, true)
	proposedText->"top"->"flush"->proposedSelected
	proposedText->"top"->"aligned"->emptyLabel
	proposedText->"left"->"flush"->updateButton
	proposedText->"bottom"->"inside"->invisibleFrame
	proposedText->"right"->"inside"->invisibleFrame
	//editable rich text
	proposedTextEdit = richText(conflictDB, "", "", 100, 160, false)
	proposedTextEdit->"top"->"flush"->proposedSelected
	proposedTextEdit->"top"->"aligned"->emptyLabel
	proposedTextEdit->"left"->"flush"->updateButton
	proposedTextEdit->"bottom"->"inside"->invisibleFrame
	proposedTextEdit->"right"->"inside"->invisibleFrame

	//create the button that allows navigation to the selected object
	DBE showObjButton = apply(conflictDB, getCsintString("csString_ConflictDetect_Button_ShowObject"), showSelectedObject)
	//create the Keep Proposed button
	DBE keepProposedButtton = apply(conflictDB, getCsintString("csString_ConflictDetect_Button_KeepProposed"), keepProposedChanges)
	//create the Discard and Finish Buttons
	DBE discardProposedButton = apply(conflictDB, getCsintString("csString_ConflictDetect_Button_DiscardProposed"), discardProposedChanges)
	finishConflictButton = apply(conflictDB, getCsintString("csString_ConflictDetect_Button_ConflictFinish"), clearObsoleteFlag)
	
	//for the Close Button
	close(conflictDB, true, onClose_ConflictDetect)

	realize conflictDB

	hide invisibleFrame
	hide invisibleFrame2
	
	//hide the stacked proposed text box
	stacked proposedTextEdit
	hide proposedTextEdit

	//set the update and cancel button inactive when dialog is first opened
	inactive updateButton
	inactive cancelButton

	//insert columns of Object with Conflicts listView
	insertColumn(objWithConflictsList, 0, getCsintString("csString_ConflictDetect_Text_ObjResolved"), 65, iconNone)
	insertColumn(objWithConflictsList, 1, getCsintString("csString_ConflictDetect_Text_ObjID"), 235, iconNone)

	//use color for Rich Text proposedText
	useRTFColour(proposedText, true)

	//function for changing selection in RadioBox
	set(proposedSelected, changeProposedText)
	//function for changing selection in "Objects with Conflicts" list
	set(objWithConflictsList, getObjectsAttributes, deselectObjWithConflict, activateObjWithConflict)
	//function for selecting checkbox in "Objects with Conflicts" list
	set(objWithConflictsList, selectObjToResolve)
	//function for changing selection in "Object Attributes" list
	set(attrWithConflictsList, displayRequiredValues)

	//call function to get all Objects with Conflicts
	getAllObjectsWithConflicts(obj_csUser, obj_csApiLib, com, curmod_uniqueID)
	int numOfObjWithConflicts = noElems(objWithConflictsList)
	if numOfObjWithConflicts < 1 then
	{
		csintDebug_("conflictDetectDialogBase(): will not show the dialog.")
		hide conflictDB
	}
	else
	{
		//check whether to set "Finish Conflict Resolution" button to active or inactive
		checkFinishButtonStatus(obj_csUser, obj_csApiLib, curmod_uniqueID)

		csintDebug_("conflictDetectDialogBase(): before showing the dialog.")
		show conflictDB
	}
	csintDebug_("conflictDetectDialogBase() end.")
}

void conflictDetectDialog(Module m, string title, string default_RCR)
{
	csintDebug_("conflictDetectDialog() start...")
	
	invalid_objSkip = createString

	//Get the current module
	Module cur_mod = current Module
	if null cur_mod then
	{
		csintDebug_("conflictDetectDialog() end.")
		return
	}
	string curmod_uniqueID = uniqueID(item(fullName(cur_mod)))

	//delete the temp file, if it exists
	objsTempFile = csintUserTempDir "\\" curmod_uniqueID "_discard.dat"
	if fileExists(objsTempFile) then
	{
		deleteFile(objsTempFile)
	}

	//Get and store the default RCR
	conflictDetect_defaultRCR = default_RCR
	//Get the obj_csUser, obj_csApiLib, com information
	OleAutoObj obj_csUser = null
	OleAutoObj obj_csApiLib = get_csApiLib(obj_csUser)
	OleAutoObj com = null
	ecpsLogin(com)

	//create the invalid objects temp file
	invalidObjTempFile = csintUserTempDir "\\" curmod_uniqueID "_sharedinvalidobj.dat"

	//Determine if the MRC is obsolete
	if (isMRCObsolete(obj_csUser, obj_csApiLib, com, curmod_uniqueID)) then
	{
		conflictDB = create(m, title, styleCentered)
		conflictDetectDialogBase(obj_csUser, obj_csApiLib, com, curmod_uniqueID)
	}
	else
	{
		if !null invalid_objSkip then delete invalid_objSkip
		if fileExists(invalidObjTempFile) then deleteFile(invalidObjTempFile)
	}
	csintDebug_("conflictDetectDialog() end.")
}